// 
//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

/// Create the input grid containing a TextField for each cell and
/// nothing more. The created grid is a MovieClip itself, and will
/// contain a two-dimensional array called 'cells' which can be
/// addressed to reach specific cells (ret.cell[x][y]).
/// Also, it will contain variables called 'cols', 'rows', 'cellWidth'
/// and 'cellHeight'
///
/// It would likely be cleaner to return an object-like thing, with
/// methods using these internal variables, but I didn't feel like going
/// there right now. If we go there, this would likely be a constructor...
///
//
/// @param where the movieclip to create the grid into
/// @param name the name to assign to the grid clip
/// @param depth the depth to assign to the grid clip
///
/// @param outputGrid
///	The output grid to make this grid an input for
///
function createInputGrid(where, name, depth, outputGrid)
{
	var cols = outputGrid.cols;
	var rows = outputGrid.rows;
	var cellWidth = outputGrid.cellWidth;
	var cellHeight = outputGrid.cellHeight;
	var grid = createEmptyMovieClip(name, depth);
	grid.cols = cols;
	grid.rows = rows;
	grid.cellWidth = cellWidth;
	grid.cellHeight = cellHeight;
	grid.cells = new Array();

	with (grid)
	{
		depth = 1;

		for (var i=0; i<cols; ++i)
		{
			var rowCells = new Array();
			var rowY = cellHeight*i;

			for (var j=0; j<rows; j++)
			{
				var name = 'cell'+i+','+j;
				createTextField(name, depth++, 0, 0, cellHeight, cellWidth);
				var cell = grid[name];
				cell.i = i; // keep track for later use
				cell.j = j; // keep track for later use
				cell.type = "input";
				cell.tabIndex = depth;
				cell.text = "0";
				cell.border = false;
				cell._y = rowY;
				cell._x = cellWidth*j;
				cell.onChanged = function()
				{
					//trace("Cell ("+this._name+") changed to "+this.text);

					// TODO: Make sure it's a number (or zero it)
					this.text = grid.validateValue(this.text);

					var x = this.i;
					var y = this.j;

					outputGrid.setCellIntensity(x, y, this.text);
				};

				cell.onSetFocus = function()
				{
					this.border = true;
				};

				cell.onKillFocus = function()
				{
					this.border = false;
				};

				rowCells[j] = cell;
			}

			cells.push(rowCells);
		}
	}

	/// Return a valid input cell value (as a number) from an input value (as a text)
	grid.validateValue = function(value)
	{
		//trace("validateValue("+value+") called");
		var tlen = value.length;
		//trace("Text length is "+tlen);
		if ( tlen > 1 )
		{
			value = value.substr(tlen-1, 1);
		}

		//value = Number(value); // gnash doesn't support this
		value = 1*value; // this is fine for gnash

		var minIntensity = 1; // should be 1
		var maxIntensity = 9; // should be 9
		if ( isNaN(value) ) value = minIntensity;
		else if ( value < minIntensity ) value = minIntensity; 
		else if ( value > maxIntensity ) value = maxIntensity; 
		return value;
	};
	
	/// Set value of a given cell.
	/// The value will be validated and will trigger outputGrid update
	/// 
	grid.setCellValue = function(x, y, value) 
	{
		var grid = this;
		var cell = grid.cells[x][y];
		//trace("grid.cells is a "+typeof(grid.cells));
		var validatedValue = grid.validateValue(value);
		//trace("Value "+value+" got validated to "+validatedValue);
		cell.text = validatedValue;
		//trace("cell.text set to "+validatedValue+" ("+cell.text+")");
		cell.onChanged();
	};

	/// Return the value written in a given cell
	grid.getCellValue = function(x, y) 
	{
		var cell = grid.cells[x][y];
		return cell.text;
	};

	return grid;
}
