// 
//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

/// Create the output grid containing a MovieClip for each cell and
/// nothing more. The created grid is a MovieClip itself, and will
/// contain a two-dimensional array called 'cells' which can be
/// addressed to reach specific cells (ret.cell[x][y]).
/// Also, it will contain variables called 'cols', 'rows', 'cellWidth'
/// and 'cellHeight'
///
/// It would likely be cleaner to return an object-like thing, with
/// methods using these internal variables, but I didn't feel like going
/// there right now. If we go there, this would likely be a constructor...
///
//
/// @param where the movieclip to create the grid into
/// @param name the name to assign to the grid clip
/// @param depth the depth to assign to the grid clip
/// @param cols number of columns in the grid
/// @param rows number of rows in the grid
/// @param cellWidth width of each cell, in pixels
/// @param cellHeight height of each cell, in pixels
///
function createOutputGrid(where, name, depth, cols, rows, cellWidth, cellHeight)
{
	var grid = createEmptyMovieClip(name, depth);
	grid.cols = cols;
	grid.rows = rows;
	grid.cellWidth = cellWidth;
	grid.cellHeight = cellHeight;
	grid.cells = new Array();

	with (grid)
	{
		depth = 1;

		for (var i=0; i<cols; ++i)
		{
			var rowCells = new Array();
			var rowY = cellHeight*i;

			for (var j=0; j<rows; j++)
			{
				var name = 'cell'+i+','+j;
				var cell = createEmptyMovieClip(name, depth++);
				cell._y = rowY;
				cell._x = cellWidth*j;

				rowCells[j] = cell;
			}

			cells.push(rowCells);
		}
	}

	grid.drawRect = function(where, width, height, color)
	{
		where.clear();
		where.lineStyle(0, color, 100);
		where.beginFill(color, 100);
		where.lineTo(0, width);
		where.lineTo(height, width);
		where.lineTo(height, 0);
		where.lineTo(0, 0);
		where.endFill();
	};

	grid.setCellColor = function(x, y, color) 
	{
		var grid = this;
		var cell = grid.cells[x][y];
		//trace("grid.cells is a "+typeof(grid.cells));
		var cellWidth = grid.cellWidth;
		var cellHeight = grid.cellHeight;
		this.drawRect(cell, cellWidth, cellHeight, color);
	};

	grid.setCellIntensity = function(x, y, intensity) 
	{
		var grid = this;
		var cell = grid.cells[x][y];
		//trace("grid.cells is a "+typeof(grid.cells));
		var cellWidth = grid.cellWidth;
		var cellHeight = grid.cellHeight;

		// See http://en.wikipedia.org/wiki/HSV_color_space#Conversion_from_HSL_to_RGB

		var H = grid.hue; 
		var S = intensity/10; // saturation
		var V = 1; // value: max

		var color = ColorMaker.HSVtoRGB(H, S, V);

		this.drawRect(cell, cellWidth, cellHeight, color);
	};

	grid.setHue = function(v)
	{
		this.hue = v;
	};

	/// Update all cell colors based on cell values found in
	/// the given input grid
	grid.updateColors = function(inputGrid)
	{
		for (var i=0; i<inputGrid.rows; ++i)
		{
			for (var j=0; j<inputGrid.cols; ++j)
			{
				this.setCellIntensity(i, j, inputGrid.getCellValue(i, j));
			}
		}
	};

	grid.setHue(0); // red by default


	return grid;
}
